/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
#include "main.h"

#include <oauth2.h>
#include <bundle.h>
#include <dlog.h>

#include "oauth2_provider_configs.h"

/* #define _THIS_PACKAGE "org.tizen.oauth2sample"
 *
 * #ifdef  LOG_TAG
 * #undef  LOG_TAG
 * #endif
 * #define LOG_TAG _THIS_PACKAGE
*/

static oauth2_manager_h mgr1 = NULL;
char *google_refresh_token = NULL;
char *popStr;
Evas_Object *win_data = NULL;
int flag = 0;

static void
popup_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *popup = data;
	evas_object_del(popup);
}

static void
create_popup(char *popup_str, void * data)
{
	Evas_Object *popup = elm_popup_add(win_data);

	if (flag == 0)
		elm_object_part_text_set(popup, "title,text", "Response");
	else if (flag == 1)
		elm_object_part_text_set(popup, "title,text", "Success");


	Evas_Object *btn;
	char str[2048] = {0,};
	strcpy(str, popup_str);
	elm_popup_align_set(popup, ELM_NOTIFY_ALIGN_FILL, 1.0);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(popup, popup_str);

	btn = elm_button_add(popup);
	elm_object_style_set(btn, "popup");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(btn, "OK");
	elm_object_part_content_set(popup, "button1", btn);
	eext_object_event_callback_add(popup,
		EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	evas_object_smart_callback_add(btn, "clicked", popup_btn_clicked_cb, popup);
	evas_object_show(popup);


	flag = 0;
	return;
}

void
bundle_cb(const char *key, const int type, const bundle_keyval_t *kv,
	void *user_data)
{
	char *display_str = (char *)user_data;

	char *err_val = NULL;
	size_t err_len = 0;
	bundle_keyval_get_basic_val((bundle_keyval_t *)kv, (void **)&err_val,
		&err_len);
	if (err_val) {
		strcat(display_str, key);
		strcat(display_str, "=");
		strcat(display_str, err_val);
		strcat(display_str, ",");
	}
}


void
token_response_cb(oauth2_response_h response, void *user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "token_response_cb");

	char *acc_token = NULL;
	oauth2_response_get_access_token(response, &acc_token);

	char *ref_token = NULL;
	oauth2_response_get_refresh_token(response, &ref_token);
	google_refresh_token = ref_token;

	long long int expires_in = 0;
	oauth2_response_get_expires_in(response, &expires_in);

	char displayStr[1024] = {0,};
	if (acc_token) {
		displayStr[0] = '\0';
		strcpy(displayStr, "access token = ");
		strcat(displayStr, acc_token);
	} else {
		oauth2_error_h e_handle =  NULL;
		char* error_msg = NULL;

		oauth2_response_get_error(response, &e_handle);
		oauth2_error_get_description(e_handle, &error_msg);

		if (error_msg)
			strcpy(displayStr, error_msg);
		else {
			int error_code = 0;
			int platform_error_code = 0;

			oauth2_error_get_code(e_handle, &error_code, &platform_error_code);

			if (error_code != 0 || platform_error_code != 0)
				sprintf(displayStr, "Error = [%d][%d]", error_code, platform_error_code);
			else
				strcpy(displayStr, "Unknown server error");
		}
	}

	if (ref_token) {
		strcat(displayStr, "\r\n");
		strcat(displayStr, "refresh token = ");
		strcat(displayStr, ref_token);
	}

	if (expires_in != 0) {
		strcat(displayStr, "\r\n");
		strcat(displayStr, "expires in = ");
		char expires_str[128] = {0};
		sprintf(expires_str, "%lld", expires_in);
		strcat(displayStr, expires_str);
	}

	create_popup(displayStr, win_data);
	return;
}


void
refresh_token_response_cb(oauth2_response_h response, void *user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "refresh_token_response_cb");

	char *acc_token = NULL;
	oauth2_response_get_access_token(response, &acc_token);

	char *ref_token = NULL;
	oauth2_response_get_refresh_token(response, &ref_token);

	if (ref_token) {
		oauth2_manager_h mgr = (oauth2_manager_h) user_data;

		oauth2_request_h request = NULL;

		int ret = oauth2_request_create(&request);

		ret = oauth2_request_set_refresh_token_url(request,
			GOOGLE_REFRESH_TOKEN_URL);

		ret = oauth2_request_set_refresh_token(request, ref_token);

		ret = oauth2_request_set_client_id(request, GOOGLE_CLIENT_ID);

		ret = oauth2_request_set_client_secret(request, GOOGLE_CLIENT_SECRET);

		ret = oauth2_request_set_grant_type(request, OAUTH2_GRANT_TYPE_REFRESH);

		ret = oauth2_request_set_client_authentication_type(request,
			OAUTH2_CLIENT_AUTHENTICATION_TYPE_REQUEST_BODY);

		if (mgr && request) {
			mgr1 = mgr;

			ret = oauth2_manager_refresh_access_token(mgr, request,
				token_response_cb, NULL);
		}
	} else {
		char displayStr[1024] = {0,};

		oauth2_error_h e_handle =  NULL;
		char* error_msg = NULL;

		oauth2_response_get_error(response, &e_handle);
		oauth2_error_get_description(e_handle, &error_msg);

		if (error_msg)
			strcpy(displayStr, error_msg);
		else {
			int error_code = 0;
			int platform_error_code = 0;

			oauth2_error_get_code(e_handle, &error_code, &platform_error_code);

			if (error_code != 0 || platform_error_code != 0)
				sprintf(displayStr, "Error = [%d][%d]", error_code, platform_error_code);
			else
				strcpy(displayStr, "Unknown server error");
		}

		create_popup(displayStr, win_data);
	}
}

void
clear_cache_and_cookies_cb(void *data, Evas_Object *obj, void *event_info)
{
	win_data = (Evas_Object *) data;

	if (mgr1 != NULL) {
		oauth2_manager_clear_cache(mgr1);
		oauth2_manager_clear_cookies(mgr1);

		mgr1 = NULL;

		flag = 1;

		popStr = "cache and cookie data cleared";
		create_popup(popStr , win_data);
		return;
	}
}

void
code_access_token_cb(oauth2_response_h response, void *user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "auth2_access_token_cb");

	char *acc_token = NULL;
	char displayStr[1024] = {0,};
	oauth2_response_get_access_token(response, &acc_token);
	if (acc_token) {
		displayStr[0] = '\0';
		strcpy(displayStr, "access token = ");
		strcat(displayStr, acc_token);
	} else {
		oauth2_error_h e_handle = NULL;

		oauth2_response_get_error(response, &e_handle);
		char* error_msg = NULL;

		oauth2_response_get_error(response, &e_handle);
		oauth2_error_get_description(e_handle, &error_msg);

		if (error_msg)
			strcpy(displayStr, error_msg);
		else {
			int error_code = 0;
			int platform_error_code = 0;

			oauth2_error_get_code(e_handle, &error_code, &platform_error_code);

			if (error_code != 0 || platform_error_code != 0)
				sprintf(displayStr, "Error = [%d][%d]", error_code, platform_error_code);
			else
				strcpy(displayStr, "Unknown server error");
		}
	}
	create_popup(displayStr, win_data);
}

void
grant_response_cb(oauth2_response_h response, void *user_data)
{
	dlog_print(DLOG_INFO, LOG_TAG, "grant_response_cb");

	char *auth_code = NULL;
	oauth2_response_get_authorization_code(response, &auth_code);
	if (auth_code) {
		oauth2_manager_h mgr = NULL;
		int ret = oauth2_manager_create(&mgr);

		oauth2_request_h request = (oauth2_request_h) user_data;

		ret = oauth2_request_set_authorization_code(request, auth_code);

		if (mgr && request) {
			ret = oauth2_manager_request_access_token(mgr,
				request, code_access_token_cb, NULL);
		}
	} else {
		char displayStr[1024] = {0,};

		oauth2_error_h e_handle =  NULL;
		char* error_msg = NULL;

		oauth2_response_get_error(response, &e_handle);
		oauth2_error_get_description(e_handle, &error_msg);

		if (error_msg)
			strcpy(displayStr, error_msg);
		else {
			int error_code = 0;
			int platform_error_code = 0;

			oauth2_error_get_code(e_handle, &error_code, &platform_error_code);

			if (error_code != 0 || platform_error_code != 0)
				sprintf(displayStr, "Error = [%d][%d]", error_code, platform_error_code);
			else
				strcpy(displayStr, "Unknown server error");
		}

		create_popup(displayStr, win_data);
	}
}


void
start_google_oauth_cb(void *data, Evas_Object *obj, void *event_info)
{
	win_data = (Evas_Object *) data;

	oauth2_manager_h mgr = NULL;
	int ret = oauth2_manager_create(&mgr);

	oauth2_request_h request = NULL;
	ret = oauth2_request_create(&request);

	ret = oauth2_request_set_auth_end_point_url(request, GOOGLE_AUTH_URL);

	ret = oauth2_request_set_token_end_point_url(request, GOOGLE_REQ_TOK_URL);

	ret = oauth2_request_set_redirection_url(request, GOOGLE_REDIRECT_URL);

	ret = oauth2_request_set_client_id(request, GOOGLE_CLIENT_ID);

	ret = oauth2_request_set_client_secret(request, GOOGLE_CLIENT_SECRET);

	ret = oauth2_request_set_scope(request, GOOGLE_SCOPE);

	ret = oauth2_request_set_response_type(request, OAUTH2_RESPONSE_TYPE_CODE);

	ret = oauth2_request_add_custom_data(request, GOOGLE_PROMPT_KEY, GOOGLE_PROMPT_VAL);

	ret = oauth2_request_add_custom_data(request, GOOGLE_ACCESS_TYPE_KEY, GOOGLE_ACCESS_TYPE_VAL);

	if (mgr && request) {
		mgr1 = mgr;

		ret = oauth2_manager_request_token(mgr, request,
			token_response_cb, NULL);
	}
}


void
start_fb_oauth_cb(void *data, Evas_Object *obj, void *event_info)
{
	oauth2_manager_h mgr = NULL;
	int ret = oauth2_manager_create(&mgr);

	win_data = (Evas_Object *) data;

	oauth2_request_h request = NULL;
	ret = oauth2_request_create(&request);

	ret = oauth2_request_set_auth_end_point_url(request, FACEBOOK_AUTH_URL);

	ret = oauth2_request_set_redirection_url(request, OAUTH2_REDIRECT_URL);

	ret = oauth2_request_set_client_id(request, FACEBOOK_CLIENT_ID);

	ret = oauth2_request_set_scope(request, FACEBOOK_SCOPE);

	ret = oauth2_request_set_response_type(request, OAUTH2_RESPONSE_TYPE_TOKEN);

	if (mgr && request) {
		mgr1 = mgr;

		ret = oauth2_manager_request_token(mgr, request,
			token_response_cb, NULL);
	}
}


void
start_google_refresh_token_cb(void *data, Evas_Object *obj, void *event_info)
{
	win_data = (Evas_Object *) data;

	oauth2_manager_h mgr = NULL;
	int ret = oauth2_manager_create(&mgr);

	oauth2_request_h request = NULL;
	ret = oauth2_request_create(&request);

	ret = oauth2_request_set_auth_end_point_url(request, GOOGLE_AUTH_URL);

	ret = oauth2_request_set_token_end_point_url(request, GOOGLE_REQ_TOK_URL);

	ret = oauth2_request_set_redirection_url(request, GOOGLE_REDIRECT_URL);

	ret = oauth2_request_set_client_id(request, GOOGLE_CLIENT_ID);

	ret = oauth2_request_set_client_secret(request, GOOGLE_CLIENT_SECRET);

	ret = oauth2_request_set_scope(request, GOOGLE_SCOPE);

	ret = oauth2_request_set_response_type(request, OAUTH2_RESPONSE_TYPE_CODE);

	ret = oauth2_request_add_custom_data(request, GOOGLE_PROMPT_KEY, GOOGLE_PROMPT_VAL);

	ret = oauth2_request_add_custom_data(request, GOOGLE_ACCESS_TYPE_KEY, GOOGLE_ACCESS_TYPE_VAL);

	if (mgr && request) {
		ret = oauth2_manager_request_token(mgr, request,
			refresh_token_response_cb, mgr);
	}
}

void
start_twitter_apponly_oauth_cb(void *data, Evas_Object *obj, void *event_info)
{
	oauth2_manager_h mgr = NULL;
	int ret = oauth2_manager_create(&mgr);

	oauth2_request_h request = NULL;
	ret = oauth2_request_create(&request);

	win_data = (Evas_Object *) data;

	ret = oauth2_request_set_auth_end_point_url(request, TWITTER_AUTH_URL);

	ret = oauth2_request_set_token_end_point_url(request, TWITTER_REQ_TOK_URL);

	ret = oauth2_request_set_redirection_url(request, OAUTH2_REDIRECT_URL);

	ret = oauth2_request_set_client_id(request, TWITTER_CONS_KEY);

	ret = oauth2_request_set_client_secret(request, TWITTER_CONS_SEC);

	ret = oauth2_request_set_grant_type(request,
		OAUTH2_GRANT_TYPE_CLIENT_CREDENTIALS);

	ret = oauth2_request_set_client_authentication_type(request,
		OAUTH2_CLIENT_AUTHENTICATION_TYPE_BASIC);

	if (mgr && request) {
		mgr1 = mgr;

		ret = oauth2_manager_request_token(mgr, request,
			token_response_cb, NULL);
	}
}


void
start_salesforce_oauth_code_cb(void *data, Evas_Object *obj, void *event_info)
{
	oauth2_manager_h mgr = NULL;
	int ret = oauth2_manager_create(&mgr);

	oauth2_request_h request = NULL;
	ret = oauth2_request_create(&request);

	win_data = (Evas_Object *) data;

	ret = oauth2_request_set_auth_end_point_url(request, SALESFORCE_AUTH_URL);

	ret = oauth2_request_set_token_end_point_url(request,
		SALESFORCE_REQ_TOK_URL);

	ret = oauth2_request_set_redirection_url(request, OAUTH2_REDIRECT_URL);

	ret = oauth2_request_set_client_id(request,	SALESFORCE_CLIENT_ID);

	ret = oauth2_request_set_client_secret(request, SALESFORCE_CLIENT_SECRET);

	ret = oauth2_request_set_grant_type(request, OAUTH2_GRANT_TYPE_PASSWORD);

	ret = oauth2_request_set_user_name(request, SALESFORCE_CLIENT_USER_NAME);

	ret = oauth2_request_set_password(request, SALESFORCE_CLIENT_PASSWORD);

	if (mgr && request) {
		mgr1 = mgr;

		ret = oauth2_manager_request_token(mgr, request,
			token_response_cb, request);
	}
}
